'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _child_process = require('child_process');

var _fs = require('fs');

var _lodash = require('lodash');

var _gitRefs = require('git-refs');

var _gitRefs2 = _interopRequireDefault(_gitRefs);

exports['default'] = function () {
  var callback = arguments[arguments.length - 1];

  if (typeof callback !== 'function') {
    throw new Error('No callback provied.');
  }

  var opts = (0, _lodash.defaults)(arguments.length > 1 && arguments[0] || {}, {
    silent: true,
    paths: ['tests', 'package.json']
  });

  var _ref = arguments.length > 2 && arguments[1] || {};

  var nextRelease = _ref.nextRelease;

  var type = nextRelease.type || opts.type;
  if (type === 'major' || type === 'initial') return callback(null);

  (0, _child_process.exec)('git fetch --tags', function (error, stdout, stderr) {
    if (error) return callback(new Error('Could not fetch tags.'));
    (0, _child_process.exec)('git describe --abbrev=0 --tags', function (descErr, stdout, stderr) {
      (0, _gitRefs2['default'])(function (err, refs) {
        if (err) return callback(new Error('Could not get refs.'));
        var cohash = refs.get(descErr ? 'HEAD' : 'tags/' + stdout.trim());
        (0, _child_process.exec)('git checkout ' + cohash + ' ' + opts.paths.join(' '), function (error, stdout, stderr) {
          if (error) return callback(new Error('Could not checkout paths.'));
          var pkg = JSON.parse((0, _fs.readFileSync)('package.json').toString());
          var tmpDep = pkg.dependencies;
          delete pkg.dependencies;
          (0, _fs.writeFileSync)('package.json', JSON.stringify(pkg, null, 2) + '\n');
          (0, _child_process.exec)('npm install', function (error, stdout, stderr) {
            if (error) return callback(new Error('Could not install dependencies.'));
            pkg.dependencies = tmpDep;
            (0, _fs.writeFileSync)('package.json', JSON.stringify(pkg, null, 2) + '\n');
            var npmTest = (0, _child_process.exec)('npm test', function (error, stdout, stderr) {
              (0, _child_process.exec)('git stash', function () {
                // Ignoring `git stash errors

                if (error) {
                  if (opts.silent) {
                    // When things go wrong we log no matter what
                    console.error(stdout);
                    console.error(stderr);
                  }
                  return callback(new Error('Old tests failed. Breaking Change detected.'));
                }

                // Congratulations. No Breaking Change!
                callback(null);
              });
            });
            if (!opts.silent) {
              npmTest.stdout.pipe(process.stdout);
              npmTest.stderr.pipe(process.stderr);
            }
          });
        });
      });
    });
  });
};

module.exports = exports['default'];